package yonyou.udh.demo;

import java.util.List;

import scala.Tuple2;

import org.apache.spark.api.java.*;
import org.apache.spark.api.java.function.Function;
import org.apache.spark.mllib.recommendation.ALS;
import org.apache.spark.mllib.recommendation.MatrixFactorizationModel;
import org.apache.spark.mllib.recommendation.Rating;
import org.apache.spark.SparkConf;

public class CollaborativeFiltering {
  public static void main(String[] args) {
    SparkConf conf = new SparkConf().setAppName("Collaborative Filtering Example");
    JavaSparkContext sc = new JavaSparkContext(conf);

    // Load and parse the data
    String path = "hdfs://udh-spark-01:8020/tmp/mllib/ratings.dat";
    JavaRDD<String> data = sc.textFile(path);
    JavaRDD<Rating> ratings = data.map(
      new Function<String, Rating>() {
        public Rating call(String s) {
          String[] sarray = s.split("::");
          return new Rating(Integer.parseInt(sarray[0]), Integer.parseInt(sarray[1]), 
                            Double.parseDouble(sarray[2]));
        }
      }
    );

    // Build the recommendation model using ALS
    int rank = 10;
    int numIterations = 20;
    MatrixFactorizationModel model = ALS.train(JavaRDD.toRDD(ratings), rank, numIterations, 0.01); 

    
    // Evaluate the model on rating data
    JavaRDD<Tuple2<Object, Object>> userProducts = ratings.map(
      new Function<Rating, Tuple2<Object, Object>>() {
        public Tuple2<Object, Object> call(Rating r) {
          return new Tuple2<Object, Object>(r.user(), r.product());
        }
      }
    );
    JavaPairRDD<Tuple2<Integer, Integer>, Double> predictions = JavaPairRDD.fromJavaRDD(
      model.predict(JavaRDD.toRDD(userProducts)).toJavaRDD().map(
        new Function<Rating, Tuple2<Tuple2<Integer, Integer>, Double>>() {
          public Tuple2<Tuple2<Integer, Integer>, Double> call(Rating r){
            return new Tuple2<Tuple2<Integer, Integer>, Double>(
              new Tuple2<Integer, Integer>(r.user(), r.product()), r.rating());
          }
        }
    ));
    
    JavaPairRDD<Tuple2<Integer,Integer>,Tuple2<Double,Double>> ratesAndPreds = 
    	      JavaPairRDD.fromJavaRDD(ratings.map(
    	        new Function<Rating, Tuple2<Tuple2<Integer, Integer>, Double>>() {
    	          public Tuple2<Tuple2<Integer, Integer>, Double> call(Rating r){
    	            return new Tuple2<Tuple2<Integer, Integer>, Double>(
    	              new Tuple2<Integer, Integer>(r.user(), r.product()), r.rating());
    	          }
    	        }
    	    )).join(predictions);
    
    JavaRDD<Tuple2<Double, Double>> ratesAndPredsValues = ratesAndPreds.values();
    
    double MSE = JavaDoubleRDD.fromRDD(ratesAndPredsValues.map(
      new Function<Tuple2<Double, Double>, Object>() {
        public Object call(Tuple2<Double, Double> pair) {
          Double err = pair._1() - pair._2();
          return err * err;
        }
      }
    ).rdd()).mean();
    
    
    System.out.println("---------------------begin--------------------------");
    List<Tuple2<Tuple2<Integer, Integer>, Tuple2<Double, Double>>> pairs = ratesAndPreds.collect();
    for(Tuple2<Tuple2<Integer, Integer>, Tuple2<Double, Double>> item:pairs){
    	System.out.println("tuple group:"+item._1()+" values:"+item._2);
    }
    System.out.println("Mean Squared Error = " + MSE);
    System.out.println("---------------------end--------------------------");
    sc.stop();
  }
}