package yonyou.udh.demo;

import java.util.List;

import scala.Tuple2;

import org.apache.spark.SparkConf;
import org.apache.spark.SparkContext;
import org.apache.spark.api.java.JavaPairRDD;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.function.Function;
import org.apache.spark.api.java.function.PairFunction;
import org.apache.spark.mllib.classification.NaiveBayes;
import org.apache.spark.mllib.classification.NaiveBayesModel;
import org.apache.spark.mllib.regression.LabeledPoint;
import org.apache.spark.mllib.util.MLUtils;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.RowFactory;
import org.apache.spark.util.Vector;

public class NativeBayesExample {

	public static void main(String[] args) {
		
		SparkConf conf = new SparkConf().setAppName("Native Bayes Example");
	    SparkContext sc = new SparkContext(conf);
	    String path = "hdfs://udh-spark-01:8020/tmp/mllib/sample_naive_bayes_data.txt";
	    
	    JavaRDD<String> lines = sc.textFile(path, 1).toJavaRDD();
	    JavaRDD<LabeledPoint> data = lines.map(new Function<String, LabeledPoint>() {
	    	public LabeledPoint call(String line){
        		return LabeledPoint.parse(line);
        	}
		});
	    
		JavaRDD<LabeledPoint>[] splits = data.randomSplit(new double[] {0.8, 0.2}, 11L);
		JavaRDD<LabeledPoint> training = splits[0].cache(); 
		JavaRDD<LabeledPoint> test = splits[1];

		final NaiveBayesModel model = NaiveBayes.train(training.rdd(), 1.0);
		JavaPairRDD<Double, Double> predictionAndLabel = 
		  test.mapToPair(new PairFunction<LabeledPoint, Double, Double>() {
		    @Override public Tuple2<Double, Double> call(LabeledPoint p) {
		      return new Tuple2<Double, Double>(model.predict(p.features()), p.label());
		    }
		  });               
		double accuracy = predictionAndLabel.filter(new Function<Tuple2<Double, Double>, Boolean>() {
		    @Override public Boolean call(Tuple2<Double, Double> pl) {
		      return pl._1().equals(pl._2());
		    }
		  }).count() / (double) test.count();
		
		System.out.println("---------------------begin--------------------------");
	    List<Tuple2<Double, Double>> pairs = predictionAndLabel.collect();
	    for(Tuple2<Double, Double> item:pairs){
	    	System.out.println("tuple predict:"+item._1()+" real:"+item._2);
	    }
	    
	    System.out.println("model accuracy:"+accuracy);
	    System.out.println("---------------------end--------------------------");
		sc.stop();
	}

}
